# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'game_settings.ui'
#
# Created by: PyQt5 UI code generator 5.15.4
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtWidgets


class Ui_GameSettings(object):
    def setupUi(self, GameSettings):
        GameSettings.setObjectName("GameSettings")
        GameSettings.resize(759, 617)
        self.game_settings_layout = QtWidgets.QVBoxLayout(GameSettings)
        self.game_settings_layout.setObjectName("game_settings_layout")
        self.title = QtWidgets.QLabel(GameSettings)
        self.title.setText("error")
        self.title.setTextInteractionFlags(QtCore.Qt.LinksAccessibleByMouse | QtCore.Qt.TextSelectableByMouse)
        self.title.setObjectName("title")
        self.game_settings_layout.addWidget(self.title)
        self.launch_settings_groupbox = QtWidgets.QGroupBox(GameSettings)
        self.launch_settings_groupbox.setObjectName("launch_settings_groupbox")
        self.launch_settings_layout = QtWidgets.QGridLayout(self.launch_settings_groupbox)
        self.launch_settings_layout.setObjectName("launch_settings_layout")
        self.skip_update = QtWidgets.QComboBox(self.launch_settings_groupbox)
        self.skip_update.setObjectName("skip_update")
        self.skip_update.addItem("")
        self.skip_update.addItem("")
        self.skip_update.addItem("")
        self.launch_settings_layout.addWidget(self.skip_update, 0, 1, 1, 1, QtCore.Qt.AlignLeft)
        self.cloud_sync = QtWidgets.QCheckBox(self.launch_settings_groupbox)
        self.cloud_sync.setText("")
        self.cloud_sync.setObjectName("cloud_sync")
        self.launch_settings_layout.addWidget(self.cloud_sync, 2, 1, 1, 1, QtCore.Qt.AlignLeft)
        self.launch_params_label = QtWidgets.QLabel(self.launch_settings_groupbox)
        self.launch_params_label.setObjectName("launch_params_label")
        self.launch_settings_layout.addWidget(self.launch_params_label, 3, 0, 1, 1, QtCore.Qt.AlignRight)
        self.wrapper_label = QtWidgets.QLabel(self.launch_settings_groupbox)
        self.wrapper_label.setObjectName("wrapper_label")
        self.launch_settings_layout.addWidget(self.wrapper_label, 4, 0, 1, 1, QtCore.Qt.AlignRight)
        self.skip_update_label = QtWidgets.QLabel(self.launch_settings_groupbox)
        self.skip_update_label.setObjectName("skip_update_label")
        self.launch_settings_layout.addWidget(self.skip_update_label, 0, 0, 1, 1, QtCore.Qt.AlignRight)
        self.wrapper_widget = QtWidgets.QWidget(self.launch_settings_groupbox)
        self.wrapper_widget.setObjectName("wrapper_widget")
        self.wrapper_layout = QtWidgets.QHBoxLayout(self.wrapper_widget)
        self.wrapper_layout.setContentsMargins(0, 0, 0, 0)
        self.wrapper_layout.setObjectName("wrapper_layout")
        self.wrapper = QtWidgets.QLineEdit(self.wrapper_widget)
        self.wrapper.setMinimumSize(QtCore.QSize(400, 0))
        self.wrapper.setObjectName("wrapper")
        self.wrapper_layout.addWidget(self.wrapper)
        self.wrapper_button = QtWidgets.QPushButton(self.wrapper_widget)
        self.wrapper_button.setObjectName("wrapper_button")
        self.wrapper_layout.addWidget(self.wrapper_button)
        self.launch_settings_layout.addWidget(self.wrapper_widget, 4, 1, 1, 1)
        self.offline_label = QtWidgets.QLabel(self.launch_settings_groupbox)
        self.offline_label.setObjectName("offline_label")
        self.launch_settings_layout.addWidget(self.offline_label, 1, 0, 1, 1, QtCore.Qt.AlignRight)
        self.offline = QtWidgets.QComboBox(self.launch_settings_groupbox)
        self.offline.setObjectName("offline")
        self.offline.addItem("")
        self.offline.addItem("")
        self.offline.addItem("")
        self.launch_settings_layout.addWidget(self.offline, 1, 1, 1, 1, QtCore.Qt.AlignLeft)
        self.launch_params_widget = QtWidgets.QWidget(self.launch_settings_groupbox)
        self.launch_params_widget.setObjectName("launch_params_widget")
        self.launch_params_layout = QtWidgets.QHBoxLayout(self.launch_params_widget)
        self.launch_params_layout.setContentsMargins(0, 0, 0, 0)
        self.launch_params_layout.setObjectName("launch_params_layout")
        self.launch_params = QtWidgets.QLineEdit(self.launch_params_widget)
        self.launch_params.setMinimumSize(QtCore.QSize(400, 0))
        self.launch_params.setObjectName("launch_params")
        self.launch_params_layout.addWidget(self.launch_params)
        self.launch_params_button = QtWidgets.QPushButton(self.launch_params_widget)
        self.launch_params_button.setObjectName("launch_params_button")
        self.launch_params_layout.addWidget(self.launch_params_button)
        self.launch_settings_layout.addWidget(self.launch_params_widget, 3, 1, 1, 1)
        self.cloud_sync_label = QtWidgets.QLabel(self.launch_settings_groupbox)
        self.cloud_sync_label.setObjectName("cloud_sync_label")
        self.launch_settings_layout.addWidget(self.cloud_sync_label, 2, 0, 1, 1, QtCore.Qt.AlignRight)
        self.game_settings_layout.addWidget(self.launch_settings_groupbox)
        self.proton_groupbox = QtWidgets.QGroupBox(GameSettings)
        self.proton_groupbox.setObjectName("proton_groupbox")
        self.proton_layout = QtWidgets.QGridLayout(self.proton_groupbox)
        self.proton_layout.setObjectName("proton_layout")
        self.proton_wrapper = QtWidgets.QComboBox(self.proton_groupbox)
        self.proton_wrapper.setObjectName("proton_wrapper")
        self.proton_wrapper.addItem("")
        self.proton_layout.addWidget(self.proton_wrapper, 0, 1, 1, 1, QtCore.Qt.AlignLeft)
        self.proton_wrapper_label = QtWidgets.QLabel(self.proton_groupbox)
        self.proton_wrapper_label.setObjectName("proton_wrapper_label")
        self.proton_layout.addWidget(self.proton_wrapper_label, 0, 0, 1, 1, QtCore.Qt.AlignRight)
        self.proton_prefix_label = QtWidgets.QLabel(self.proton_groupbox)
        self.proton_prefix_label.setObjectName("proton_prefix_label")
        self.proton_layout.addWidget(self.proton_prefix_label, 1, 0, 1, 1, QtCore.Qt.AlignRight)
        spacerItem = QtWidgets.QSpacerItem(20, 0, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Maximum)
        self.proton_layout.addItem(spacerItem, 4, 1, 1, 1)
        self.proton_prefix_layout = QtWidgets.QVBoxLayout()
        self.proton_prefix_layout.setObjectName("proton_prefix_layout")
        self.proton_layout.addLayout(self.proton_prefix_layout, 1, 1, 2, 1)
        self.proton_prefix_alignment_label = QtWidgets.QLabel(self.proton_groupbox)
        self.proton_prefix_alignment_label.setEnabled(False)
        self.proton_prefix_alignment_label.setText("")
        self.proton_prefix_alignment_label.setObjectName("proton_prefix_alignment_label")
        self.proton_layout.addWidget(self.proton_prefix_alignment_label, 2, 0, 1, 1)
        self.linux_layout = QtWidgets.QVBoxLayout()
        self.linux_layout.setSpacing(0)
        self.linux_layout.setObjectName("linux_layout")
        self.proton_layout.addLayout(self.linux_layout, 3, 0, 1, 2)
        self.game_settings_layout.addWidget(self.proton_groupbox)
        spacerItem1 = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.game_settings_layout.addItem(spacerItem1)

        self.retranslateUi(GameSettings)
        QtCore.QMetaObject.connectSlotsByName(GameSettings)

    def retranslateUi(self, GameSettings):
        _translate = QtCore.QCoreApplication.translate
        GameSettings.setWindowTitle(_translate("GameSettings", "GameSettings"))
        self.launch_settings_groupbox.setTitle(_translate("GameSettings", "Launch Settings"))
        self.skip_update.setItemText(0, _translate("GameSettings", "Default"))
        self.skip_update.setItemText(1, _translate("GameSettings", "Yes"))
        self.skip_update.setItemText(2, _translate("GameSettings", "No"))
        self.launch_params_label.setText(_translate("GameSettings", "Launch parameters"))
        self.wrapper_label.setText(_translate("GameSettings", "Wrapper"))
        self.skip_update_label.setText(_translate("GameSettings", "Skip update check"))
        self.wrapper.setPlaceholderText(_translate("GameSettings", "e.g. optirun"))
        self.wrapper_button.setText(_translate("GameSettings", "Save"))
        self.offline_label.setText(_translate("GameSettings", "Offline mode"))
        self.offline.setItemText(0, _translate("GameSettings", "Default"))
        self.offline.setItemText(1, _translate("GameSettings", "Yes"))
        self.offline.setItemText(2, _translate("GameSettings", "No"))
        self.launch_params.setPlaceholderText(_translate("GameSettings", "parameters"))
        self.launch_params_button.setText(_translate("GameSettings", "Save"))
        self.cloud_sync_label.setText(_translate("GameSettings", "Sync with cloud"))
        self.proton_groupbox.setTitle(_translate("GameSettings", "Linux Settings"))
        self.proton_wrapper.setItemText(0, _translate("GameSettings", "Don\'t use Proton"))
        self.proton_wrapper_label.setText(_translate("GameSettings", "Proton"))
        self.proton_prefix_label.setText(_translate("GameSettings", "Prefix"))


if __name__ == "__main__":
    import sys

    app = QtWidgets.QApplication(sys.argv)
    GameSettings = QtWidgets.QWidget()
    ui = Ui_GameSettings()
    ui.setupUi(GameSettings)
    GameSettings.show()
    sys.exit(app.exec_())



import webbrowser

from PyQt5.QtWidgets import QWidget

from rare import __version__
from rare.ui.components.tabs.settings.about import Ui_About
from rare.utils.utils import get_latest_version


def versiontuple(v):
    try:
        return tuple(map(int, (v.split("."))))
    except:
        return tuple((9, 9, 9))  # It is a beta version and newer


class About(QWidget, Ui_About):
    def __init__(self):
        super(About, self).__init__()
        self.setupUi(self)

        self.version.setText(__version__)

        self.update_label.setVisible(False)
        self.update.setVisible(False)
        self.open_browser.setVisible(False)

        latest_tag = get_latest_version()
        self.update_available = versiontuple(latest_tag) > versiontuple(__version__)

        self.update.setText("{} -> {}".format(__version__, latest_tag))

        self.open_browser.clicked.connect(
            lambda: webbrowser.open("https://github.com/Dummerle/Rare/releases/latest"))

        if self.update_available:
            print(f"Update available: {__version__} -> {latest_tag}")
            self.update_label.setVisible(True)
            self.update.setVisible(True)
            self.open_browser.setVisible(True)



import os
from logging import getLogger

from PyQt5.QtCore import QProcess, pyqtSignal, Qt
from PyQt5.QtWidgets import QHBoxLayout, QLabel, QPushButton, QVBoxLayout
from qtawesome import icon

from custom_legendary.core import LegendaryCore
from custom_legendary.models.game import InstalledGame
from rare.components.tabs.games.game_widgets.base_installed_widget import BaseInstalledWidget

logger = getLogger("GameWidget")


class InstalledListWidget(BaseInstalledWidget):
    proc: QProcess
    signal = pyqtSignal(str)
    update_game = pyqtSignal()

    def __init__(self, game: InstalledGame, core: LegendaryCore, pixmap, offline):
        super(InstalledListWidget, self).__init__(game, core, pixmap, offline)
        self.dev = core.get_game(self.igame.app_name).metadata["developer"]
        self.size = game.install_size
        self.launch_params = game.launch_parameters

        self.layout = QHBoxLayout()

        ##Layout on the right
        self.childLayout = QVBoxLayout()

        if self.pixmap:
            self.pixmap = self.pixmap.scaled(180, 240, transformMode=Qt.SmoothTransformation)
            self.image = QLabel()
            self.image.setPixmap(self.pixmap)
            self.layout.addWidget(self.image)

        play_icon = icon("ei.play", color="white")
        self.title_widget = QLabel(f"<h1>{self.igame.title}</h1>")
        self.app_name_label = QLabel(self.igame.app_name)
        self.launch_button = QPushButton(play_icon, self.tr("Launch"))
        self.launch_button.setObjectName("launch_game_button")
        self.launch_button.setFixedWidth(120)

        self.info = QPushButton("Info")
        self.info.clicked.connect(lambda: self.show_info.emit(self.igame.app_name))
        self.info.setFixedWidth(80)

        self.launch_button.clicked.connect(self.launch)
        self.developer_label = QLabel(self.tr("Developer: ") + self.dev)
        self.version_label = QLabel("Version: " + str(self.igame.version))
        self.size_label = QLabel(f"{self.tr('Installed size')}: {round(self.size / (1024 ** 3), 2)} GB")
        self.childLayout.addWidget(self.title_widget)
        self.childLayout.addWidget(self.launch_button)
        self.childLayout.addWidget(self.info)
        self.childLayout.addWidget(self.app_name_label)
        self.childLayout.addWidget(self.developer_label)

        self.childLayout.addWidget(self.version_label)
        self.childLayout.addWidget(self.size_label)

        self.info_label = QLabel("")
        self.childLayout.addWidget(self.info_label)
        # self.childLayout.addWidget(QPushButton("Settings"))
        # self.childLayout.addWidget(QPushButton("Uninstall"))
        self.childLayout.addStretch(1)
        self.layout.addLayout(self.childLayout)
        self.layout.addStretch(1)
        self.setLayout(self.layout)

    def launch(self):
        if not self.game_running:
            super(InstalledListWidget, self).launch(skip_version_check=self.update_available)



import os
from logging import getLogger

from PyQt5.QtCore import Qt, QThread, pyqtSignal
from PyQt5.QtWidgets import QDialog
from requests.exceptions import ConnectionError

from custom_legendary.core import LegendaryCore
from rare import image_dir
from rare.components.dialogs.login import LoginDialog
from rare.ui.components.dialogs.launch_dialog import Ui_LaunchDialog
from rare.utils.utils import download_images

logger = getLogger("Login")


class ImageThread(QThread):
    download_progess = pyqtSignal(int)

    def __init__(self, core: LegendaryCore, parent=None):
        super(ImageThread, self).__init__(parent)
        self.core = core

    def run(self):
        download_images(self.download_progess, self.core)
        self.download_progess.emit(100)


class LaunchDialog(QDialog, Ui_LaunchDialog):
    quit_app = pyqtSignal(int)
    start_app = pyqtSignal(bool)

    def __init__(self, core: LegendaryCore, offline=False, parent=None):
        super(LaunchDialog, self).__init__(parent=parent)
        self.setupUi(self)
        self.setAttribute(Qt.WA_DeleteOnClose, True)
        self.core = core
        self.offline = offline
        self.image_thread = None

    def login(self):
        do_launch = True
        try:
            if self.offline:
                pass
            else:
                if self.core.login():
                    logger.info("You are logged in")
                else:
                    raise ValueError("You are not logged in. Open Login Window")
        except ValueError as e:
            logger.info(str(e))
            do_launch = LoginDialog(core=self.core, parent=self).login()
        except ConnectionError as e:
            logger.warning(e)
            self.offline = True
        finally:
            if do_launch:
                self.show()
                self.launch()
            else:
                self.quit_app.emit(0)

    def launch(self):
        # self.core = core
        if not os.path.exists(image_dir):
            os.makedirs(image_dir)

        if not self.offline:
            self.image_info.setText(self.tr("Downloading Images"))
            self.image_thread = ImageThread(self.core, self)
            self.image_thread.download_progess.connect(self.update_image_progbar)
            self.image_thread.finished.connect(self.finish)
            self.image_thread.finished.connect(lambda: self.image_info.setText(self.tr("Ready")))
            self.image_thread.finished.connect(self.image_thread.quit)
            self.image_thread.finished.connect(self.image_thread.deleteLater)
            self.image_thread.start()
        else:
            self.finish()

    def update_image_progbar(self, i: int):
        self.image_prog_bar.setValue(i)

    def finish(self):
        self.image_info.setText(self.tr("Starting..."))
        self.image_prog_bar.setValue(100)
        self.start_app.emit(self.offline)



import json
import os
import platform

from PyQt5.QtCore import Qt, pyqtSignal
from PyQt5.QtGui import QKeyEvent
from PyQt5.QtWidgets import QWidget, QTabWidget, QTreeView
from qtawesome import icon

from custom_legendary.core import LegendaryCore
from custom_legendary.models.game import Game
from rare.ui.components.tabs.games.game_info.game_info import Ui_GameInfo
from rare.utils.extra_widgets import SideTabBar
from rare.utils.json_formatter import QJsonModel
from rare.utils.steam_grades import SteamWorker
from rare.utils.utils import get_pixmap


class UninstalledTabInfo(QTabWidget):
    def __init__(self, core, parent):
        super(UninstalledTabInfo, self).__init__(parent=parent)
        self.app_name = ""
        self.core = core
        self.setTabBar(SideTabBar())
        self.setTabPosition(QTabWidget.West)

        self.addTab(QWidget(), icon("mdi.keyboard-backspace", color="white"), self.tr("Back"))
        self.tabBarClicked.connect(lambda x: self.parent().layout.setCurrentIndex(0) if x == 0 else None)

        self.info = UninstalledInfo(core, self)
        self.addTab(self.info, self.tr("Game Info"))

        self.view = QTreeView()
        self.view.setColumnWidth(0, 300)
        self.view.setWordWrap(True)

        self.model = QJsonModel()
        self.view.setModel(self.model)

        self.addTab(self.view, self.tr("Metadata"))

        # self.setTabEnabled(1, False)
        self.setCurrentIndex(1)

    def update_game(self, app_name):
        self.info.update_game(app_name)
        self.model.clear()
        self.model.load(json.load(open(os.path.expanduser(f"~/.config/legendary/metadata/{app_name}.json"), "r")))

    def keyPressEvent(self, e: QKeyEvent):
        if e.key() == Qt.Key_Escape:
            self.parent().layout.setCurrentIndex(0)


class UninstalledInfo(QWidget, Ui_GameInfo):
    game: Game
    install_game = pyqtSignal(str)

    def __init__(self, core: LegendaryCore, parent=None):
        super(UninstalledInfo, self).__init__(parent=parent)
        self.setupUi(self)
        self.core = core

        if platform.system() != "Windows":
            self.steam_worker = SteamWorker(self.core)
            self.steam_worker.rating_signal.connect(self.grade.setText)

        if platform.system() == "Windows":
            self.lbl_grade.setVisible(False)
            self.grade.setVisible(False)

        self.install_size.setEnabled(False)
        self.lbl_install_size.setEnabled(False)
        self.install_path.setEnabled(False)
        self.lbl_install_path.setEnabled(False)

        self.game_actions_stack.setCurrentIndex(1)
        self.game_actions_stack.resize(self.game_actions_stack.minimumSize())

        self.install_button.clicked.connect(lambda: self.install_game.emit(self.game.app_name))

    def update_game(self, app_name):
        self.game = self.core.get_game(app_name)
        self.game_title.setText(f"<h2>{self.game.app_title}</h2>")

        pixmap = get_pixmap(app_name)
        w = 200
        pixmap = pixmap.scaled(w, int(w * 4 / 3))
        self.image.setPixmap(pixmap)

        self.app_name.setText(self.game.app_name)
        self.version.setText(self.game.app_version)
        self.dev.setText(self.game.metadata["developer"])
        self.install_size.setText("N/A")
        self.install_path.setText("N/A")

        if platform.system() != "Windows":
            self.grade.setText(self.tr("Loading"))
            self.steam_worker.set_app_name(app_name)
            self.steam_worker.start()



import platform
import time
from logging import getLogger

import pypresence.exceptions
from PyQt5.QtCore import QObject, QSettings
from pypresence import Presence

from custom_legendary.core import LegendaryCore

client_id = "830732538225360908"
logger = getLogger("RPC")


class DiscordRPC(QObject):
    def __init__(self, core: LegendaryCore):
        super(DiscordRPC, self).__init__()
        self.RPC = None
        self.state = 1  # 0: game, 1: always active, 2: off
        self.core = core
        self.settings = QSettings()
        if self.settings.value("rpc_enable", 0, int) == 1:  # show always
            self.state = 2
            self.set_discord_rpc()

    def changed_settings(self, game_running: list):
        value = self.settings.value("rpc_enable", 0, int)
        if value == 2:
            self.remove_rpc()
            return
        if not game_running and value == 0:
            self.remove_rpc()
        elif not game_running:
            self.set_discord_rpc()
        else:
            self.set_discord_rpc(game_running[0])

    def remove_rpc(self):
        if self.settings.value("rpc_enable", 0, int) != 1:
            if not self.RPC:
                return
            try:
                self.RPC.close()
            except Exception:
                logger.warning("Already closed")
            del self.RPC
            self.RPC = None
            logger.info("Remove RPC")
            self.state = 2
        else:
            self.state = 1
            self.set_discord_rpc()

    def set_discord_rpc(self, app_name=None):
        if not self.RPC:
            try:
                self.RPC = Presence("830732538225360908")  # Rare app: https://discord.com/developers/applications
                self.RPC.connect()
            except ConnectionRefusedError as e:
                logger.warning("Discord is not active\n" + str(e))
                self.RPC = None
                return
            except FileNotFoundError as e:
                logger.warning("File not found error\n" + str(e))
                self.RPC = None
                return
            except pypresence.exceptions.InvalidPipe as e:
                logger.error("Is Discord running? \n" + str(e))
                self.RPC = None
                return
            except Exception as e:
                logger.error(str(e))
                self.RPC = None
                return
        self.update_rpc(app_name)

    def update_rpc(self, app_name=None):
        if self.settings.value("rpc_enable", 0, int) == 2 or (
                app_name is None and self.settings.value("rpc_enable", 0, int) == 0):
            self.remove_rpc()
            return
        title = None
        if not app_name:
            self.RPC.update(large_image="logo",
                            details="https://github.com/Dummerle/Rare")
            return
        if self.settings.value("rpc_name", True, bool):
            title = self.core.get_installed_game(app_name).title
        start = None
        if self.settings.value("rpc_time", True, bool):
            start = str(time.time()).split(".")[0]
        os = None
        if self.settings.value("rpc_os", True, bool):
            os = "via Rare on " + platform.system()

        self.RPC.update(large_image="logo",
                        details=title,
                        large_text=title,
                        state=os,
                        start=start)
        self.state = 0



from logging import getLogger

from PyQt5.QtCore import QEvent, pyqtSignal, QSize, Qt
from PyQt5.QtGui import QMouseEvent
from PyQt5.QtWidgets import *
from qtawesome import icon

from custom_legendary.core import LegendaryCore
from custom_legendary.models.game import InstalledGame
from rare.components.tabs.games.game_widgets.base_installed_widget import BaseInstalledWidget
from rare.utils.extra_widgets import ClickableLabel

logger = getLogger("GameWidgetInstalled")


class GameWidgetInstalled(BaseInstalledWidget):
    update_list = pyqtSignal(str)
    show_info = pyqtSignal(str)
    update_game = pyqtSignal()

    def __init__(self, game: InstalledGame, core: LegendaryCore, pixmap, offline):
        super(GameWidgetInstalled, self).__init__(game, core, pixmap, offline)
        self.setObjectName("game_widget_icon")

        self.setContextMenuPolicy(Qt.ActionsContextMenu)

        self.layout = QVBoxLayout()
        self.core = core
        self.running = False
        self.info_text = ""

        if self.update_available:
            logger.info("Update available for game: " + self.game.app_name)
            self.info_text = self.tr("Update available")

        if self.pixmap:
            w = 200
            self.pixmap = self.pixmap.scaled(w, int(w * 4 / 3), transformMode=Qt.SmoothTransformation)
            self.image = ClickableLabel()
            self.image.setObjectName("game_widget")
            self.image.setPixmap(self.pixmap)
            self.layout.addWidget(self.image)

        self.title_label = QLabel(f"<h4>{game.title}</h4>")
        self.title_label.setAutoFillBackground(False)
        self.title_label.setWordWrap(True)
        self.title_label.setFixedWidth(175)
        minilayout = QHBoxLayout()
        self.title_label.setObjectName("game_widget")
        minilayout.addWidget(self.title_label)

        # Info Button
        self.menu_btn = QPushButton()
        self.menu_btn.setIcon(icon("ei.info-circle", color="white"))
        # self.menu_btn.setObjectName("installed_menu_button")
        self.menu_btn.setIconSize(QSize(18, 18))
        self.menu_btn.enterEvent = lambda x: self.info_label.setText("Information")
        self.menu_btn.leaveEvent = lambda x: self.info_label.setText(
            "Please update Game") if self.update_available else self.info_label.setText("Start Game")
        # remove Border

        self.menu_btn.setObjectName("menu_button")

        self.menu_btn.clicked.connect(lambda: self.show_info.emit(self.game.app_name))
        self.menu_btn.setFixedWidth(17)
        minilayout.addWidget(self.menu_btn)
        minilayout.addStretch(1)
        self.layout.addLayout(minilayout)

        self.info_label = QLabel(self.info_text)
        self.info_label.setAutoFillBackground(False)
        self.info_label.setObjectName("info_label")
        self.layout.addWidget(self.info_label)

        # p = self.palette()
        # p.setColor(self.backgroundRole(), Qt.red)
        # self.setPalette(p)

        self.setLayout(self.layout)
        self.setFixedWidth(self.sizeHint().width())

    def enterEvent(self, a0: QEvent) -> None:
        if self.game_running:
            self.info_label.setText(self.tr("Game running"))
        elif self.update_available:
            self.info_label.setText(self.tr("Start game without version check"))
        else:
            self.info_label.setText("Start Game")

    def leaveEvent(self, a0: QEvent) -> None:
        if self.running:
            self.info_label.setText(self.tr("Game running"))
        else:
            self.info_label.setText(self.info_text)

    def mousePressEvent(self, e: QMouseEvent):
        # left button
        if e.button() == 1 and not self.game_running:
            if self.update_available:
                self.launch(skip_version_check=True)
            else:
                self.launch()

        # right
        elif e.button() == 2:
            pass  # self.showMenu(e)

    """def showMenu(self, event):
        menu = QMenu()
        desktop_link = menu.addAction("Add Desktop link")
        action = menu.exec_(self.mapToGlobal(event.pos()))
        if action == desktop_link:
            print("LOL")"""



import os
import platform

from PyQt5.QtCore import Qt, pyqtSignal
from PyQt5.QtGui import QKeyEvent
from PyQt5.QtWidgets import QWidget, QTabWidget, QMessageBox
from qtawesome import icon

from custom_legendary.core import LegendaryCore
from custom_legendary.models.game import Game, InstalledGame
from rare.components.tabs.games.game_info.dlcs import DlcTab
from rare.components.tabs.games.game_info.game_settings import GameSettings
from rare.ui.components.tabs.games.game_info.game_info import Ui_GameInfo
from rare.utils.extra_widgets import SideTabBar
from rare.utils.legendary_utils import VerifyThread
from rare.utils.steam_grades import SteamWorker
from rare.utils.utils import get_size, get_pixmap


class InfoTabs(QTabWidget):
    def __init__(self, core, parent):
        super(InfoTabs, self).__init__(parent=parent)
        self.app_name = ""
        self.core = core
        self.setTabBar(SideTabBar())
        self.setTabPosition(QTabWidget.West)

        self.addTab(QWidget(), icon("mdi.keyboard-backspace", color="white"), self.tr("Back"))
        self.tabBarClicked.connect(lambda x: self.parent().layout.setCurrentIndex(0) if x == 0 else None)

        self.info = GameInfo(core, self)
        self.addTab(self.info, self.tr("Game Info"))

        self.settings = GameSettings(core, self)
        self.addTab(self.settings, self.tr("Settings"))
        self.tabBar().setCurrentIndex(1)

        self.dlc_tab = DlcTab(core, self)
        self.addTab(self.dlc_tab, self.tr("DLCs"))

    def update_game(self, app_name, dlcs: list):

        self.info.update_game(app_name)
        self.settings.update_game(app_name)

        # DLC Tab: Disable if no dlcs available
        if dlcs:
            if len(dlcs[self.core.get_game(app_name).asset_info.catalog_item_id]) == 0:
                self.setTabEnabled(3, False)
            else:
                self.setTabEnabled(3, True)
                self.dlc_tab.update_dlcs(app_name, dlcs)
        else:
            self.setTabEnabled(3, False)

    def keyPressEvent(self, e: QKeyEvent):
        if e.key() == Qt.Key_Escape:
            self.parent().layout.setCurrentIndex(0)



class GameInfo(QWidget, Ui_GameInfo):
    igame: InstalledGame
    game: Game
    uninstall_game = pyqtSignal(str)
    update_list = pyqtSignal(str)
    verify_game = pyqtSignal(str)
    verify_threads = {}

    def __init__(self, core: LegendaryCore, parent):
        super(GameInfo, self).__init__(parent=parent)
        self.setupUi(self)
        self.core = core

        if platform.system() == "Windows":
            self.lbl_grade.setVisible(False)
            self.grade.setVisible(False)

        if platform.system() != "Windows":
            self.steam_worker = SteamWorker(self.core)
            self.steam_worker.rating_signal.connect(self.grade.setText)

        self.game_actions_stack.setCurrentIndex(0)
        self.game_actions_stack.resize(self.game_actions_stack.minimumSize())

        self.uninstall_button.clicked.connect(self.uninstall)
        self.verify_button.clicked.connect(self.verify)
        self.repair_button.clicked.connect(self.repair)

    def uninstall(self):
        self.uninstall_game.emit(self.game.app_name)
        self.update_list.emit(self.game.app_name)

    def repair(self):
        repair_file = os.path.join(self.core.lgd.get_tmp_path(), f'{self.game.app_name}.repair')
        if not os.path.exists(repair_file):
            QMessageBox.warning(self, "Warning", self.tr(
                "Repair file does not exist or game does not need a repair. Please verify game first"))
            return
        self.verify_game.emit(self.game.app_name)

    def verify(self):
        self.verify_widget.setCurrentIndex(1)
        verify_thread = VerifyThread(self.core, self.game.app_name)
        verify_thread.status.connect(self.verify_satistics)
        verify_thread.summary.connect(self.finish_verify)
        verify_thread.start()
        self.verify_progress.setValue(0)
        self.verify_threads[self.game.app_name] = verify_thread

    def verify_satistics(self, progress):
        # checked, max, app_name
        if progress[2] == self.game.app_name:
            self.verify_progress.setValue(progress[0] * 100 / progress[1])

    def finish_verify(self, failed):
        failed, missing, app_name = failed
        if failed == 0 and missing == 0:
            QMessageBox.information(self, "Summary",
                                    "Game was verified successfully. No missing or corrupt files found")
        else:
            ans = QMessageBox.question(self, "Summary", self.tr(
                'Verification failed, {} file(s) corrupted, {} file(s) are missing. Do you want to repair them?').format(
                failed, missing), QMessageBox.Yes | QMessageBox.No, QMessageBox.Yes)
            if ans == QMessageBox.Yes:
                self.verify_game.emit(self.game.app_name)
        self.verify_widget.setCurrentIndex(0)
        self.verify_threads.pop(app_name)

    def update_game(self, app_name):
        self.game = self.core.get_game(app_name)
        self.igame = self.core.get_installed_game(app_name)
        self.game_title.setText(f"<h2>{self.game.app_title}</h2>")

        pixmap = get_pixmap(app_name)
        w = 200
        pixmap = pixmap.scaled(w, int(w * 4 / 3))
        self.image.setPixmap(pixmap)

        self.app_name.setText(self.game.app_name)
        self.version.setText(self.game.app_version)
        self.dev.setText(self.game.metadata["developer"])
        self.install_size.setText(get_size(self.igame.install_size))
        self.install_path.setText(self.igame.install_path)

        if platform.system() != "Windows":
            self.grade.setText(self.tr("Loading"))
            self.steam_worker.set_app_name(app_name)
            self.steam_worker.start()

        if len(self.verify_threads.keys()) == 0 or not self.verify_threads.get(app_name):
            self.verify_widget.setCurrentIndex(0)
        elif self.verify_threads.get(app_name):
            self.verify_widget.setCurrentIndex(1)
            self.verify_progress.setValue(
                self.verify_threads[app_name].num / self.verify_threads[app_name].total * 100
            )



import webbrowser

from PyQt5.QtCore import QSize, pyqtSignal
from PyQt5.QtWidgets import QMenu, QTabWidget, QWidget, QWidgetAction
from qtawesome import icon

from custom_legendary.core import LegendaryCore
from rare.components.dialogs.install_dialog import InstallDialog
from rare.components.dialogs.uninstall_dialog import UninstallDialog
from rare.components.tab_utils import TabBar, TabButtonWidget
from rare.components.tabs.account import MiniWidget
from rare.components.tabs.cloud_saves import SyncSaves
from rare.components.tabs.downloads import DownloadTab
from rare.components.tabs.games import GameTab
from rare.components.tabs.settings import SettingsTab
from rare.utils import legendary_utils
from rare.utils.models import InstallQueueItemModel, InstallOptionsModel


class TabWidget(QTabWidget):
    quit_app = pyqtSignal(int)
    delete_presence = pyqtSignal()

    def __init__(self, core: LegendaryCore, parent, offline):
        super(TabWidget, self).__init__(parent=parent)
        disabled_tab = 3 if not offline else 1
        self.core = core
        self.setTabBar(TabBar(disabled_tab))

        # Generate Tabs
        self.games_tab = GameTab(core, self, offline)
        self.addTab(self.games_tab, self.tr("Games"))

        if not offline:
            updates = self.games_tab.default_widget.game_list.updates
            self.downloadTab = DownloadTab(core, updates, self)
            self.addTab(self.downloadTab, "Downloads" + (" (" + str(len(updates)) + ")" if len(updates) != 0 else ""))

            self.cloud_saves = SyncSaves(core, self)
            self.addTab(self.cloud_saves, "Cloud Saves")

        self.settings = SettingsTab(core, self)

        # Space Tab
        self.addTab(QWidget(), "")
        self.setTabEnabled(disabled_tab, False)
        # Buttons
        store_button = TabButtonWidget(core, 'fa.shopping-cart', 'Epic Games Store')
        store_button.pressed.connect(lambda: webbrowser.open("https://www.epicgames.com/store"))
        self.tabBar().setTabButton(disabled_tab, self.tabBar().RightSide, store_button)

        self.account = QWidget()
        self.addTab(self.account, "")
        self.setTabEnabled(disabled_tab + 1, False)

        self.mini_widget = MiniWidget(core)
        self.mini_widget.quit_app.connect(self.quit_app.emit)
        account_action = QWidgetAction(self)
        account_action.setDefaultWidget(self.mini_widget)
        account_button = TabButtonWidget(core, 'mdi.account-circle', 'Account')
        account_button.setMenu(QMenu())
        account_button.menu().addAction(account_action)
        self.tabBar().setTabButton(disabled_tab + 1, self.tabBar().RightSide, account_button)

        self.addTab(self.settings, icon("fa.gear", color='white'),
                    "(!)" if self.settings.about.update_available else "")

        # Signals
        # open download tab
        self.games_tab.default_widget.game_list.update_game.connect(lambda: self.setCurrentIndex(1))

        # uninstall
        self.games_tab.game_info.info.uninstall_game.connect(self.uninstall_game)

        # imported
        self.games_tab.import_widget.update_list.connect(self.game_imported)

        if not offline:
            # Download finished
            self.downloadTab.finished.connect(self.dl_finished)
            # show uninstalled info
            self.games_tab.default_widget.game_list.show_uninstalled_info.connect(self.games_tab.show_uninstalled)
            # install dlc
            self.games_tab.game_info.dlc_tab.install_dlc.connect(
                lambda app_name, update: self.install_game(
                    InstallOptionsModel(app_name=app_name),
                    update=update))

            # install game
            self.games_tab.uninstalled_info_widget.info.install_game.connect(
                lambda app_name: self.install_game(
                    InstallOptionsModel(app_name=app_name)))
            # repair game
            self.games_tab.game_info.info.verify_game.connect(
                lambda app_name: self.install_game(
                    InstallOptionsModel(app_name=app_name,
                                        base_path=core.get_installed_game(app_name).install_path,
                                        repair=True),
                    silent=True)
            )

            # Finished sync
            self.cloud_saves.finished.connect(self.finished_sync)
        # Game finished
        self.games_tab.default_widget.game_list.game_exited.connect(self.game_finished)

        # Open game list on click on Games tab button
        self.tabBarClicked.connect(lambda x: self.games_tab.layout.setCurrentIndex(0) if x == 0 else None)
        self.setIconSize(QSize(25, 25))

    # TODO; maybe pass InstallOptionsModel only, not split arguments
    def install_game(self, options: InstallOptionsModel, update=False, silent=False):
        install_dialog = InstallDialog(self.core,
                                       InstallQueueItemModel(options=options),
                                       update=update, silent=silent, parent=self)
        install_dialog.result_ready.connect(self.on_install_dialog_closed)
        install_dialog.execute()

    def on_install_dialog_closed(self, download_item: InstallQueueItemModel):
        if download_item:
            self.setCurrentIndex(1)
            self.start_download(download_item)

    def start_download(self, download_item: InstallQueueItemModel):
        downloads = len(self.downloadTab.dl_queue) + len(self.downloadTab.update_widgets.keys()) + 1
        self.setTabText(1, "Downloads" + ((" (" + str(downloads) + ")") if downloads != 0 else ""))
        self.setCurrentIndex(1)
        self.downloadTab.install_game(download_item)

    def game_imported(self, app_name: str):
        igame = self.core.get_installed_game(app_name)
        if self.core.get_asset(app_name, True).build_version != igame.version:
            self.downloadTab.add_update(igame)
            downloads = len(self.downloadTab.dl_queue) + len(self.downloadTab.update_widgets.keys())
            self.setTabText(1, "Downloads" + ((" (" + str(downloads) + ")") if downloads != 0 else ""))
        self.games_tab.default_widget.game_list.update_list(app_name)
        self.games_tab.layout.setCurrentIndex(0)

    # Sync game and delete dc rpc
    def game_finished(self, app_name):
        self.delete_presence.emit()
        if self.core.get_game(app_name).supports_cloud_saves:
            self.cloud_saves.sync_game(app_name, True)

    def uninstall_game(self, app_name):
        game = self.core.get_game(app_name)
        infos = UninstallDialog(game).get_information()
        if infos == 0:
            return
        legendary_utils.uninstall(game.app_name, self.core, infos)
        if app_name in self.downloadTab.update_widgets.keys():
            self.downloadTab.update_layout.removeWidget(self.downloadTab.update_widgets[app_name])
            self.downloadTab.update_widgets.pop(app_name)
            downloads = len(self.downloadTab.dl_queue) + len(self.downloadTab.update_widgets.keys())
            self.setTabText(1, "Downloads" + ((" (" + str(downloads) + ")") if downloads != 0 else ""))
            self.downloadTab.update_text.setVisible(len(self.downloadTab.update_widgets) == 0)

    # Update gamelist and set text of Downlaods to "Downloads"

    def dl_finished(self, update_list):
        if update_list[0]:
            self.games_tab.default_widget.game_list.update_list(update_list[1])
            self.cloud_saves.reload(update_list[1])
        downloads = len(self.downloadTab.dl_queue) + len(self.downloadTab.update_widgets.keys())
        self.setTabText(1, "Downloads" + ((" (" + str(downloads) + ")") if downloads != 0 else ""))

    def resizeEvent(self, event):
        self.tabBar().setMinimumWidth(self.width())
        super(TabWidget, self).resizeEvent(event)

    # Remove text "sync game"
    def finished_sync(self, app_name):
        self.games_tab.default_widget.game_list.widgets[app_name][0].info_text = ""
        self.games_tab.default_widget.game_list.widgets[app_name][0].info_label.setText("")
        self.games_tab.default_widget.game_list.widgets[app_name][1].info_label.setText("")



import os

from PyQt5.QtCore import pyqtSignal, QSettings
from PyQt5.QtGui import QPixmap
from PyQt5.QtWidgets import QGroupBox, QHBoxLayout, QVBoxLayout, QScrollArea, QLabel, QPushButton, QMessageBox

from custom_legendary.core import LegendaryCore
from custom_legendary.models.game import Game
from rare.utils.utils import download_image


class DlcTab(QScrollArea):
    install_dlc = pyqtSignal(str, bool)
    game: Game

    def __init__(self, core: LegendaryCore, parent):
        super(DlcTab, self).__init__(parent=parent)
        self.core = core
        self.widget = QGroupBox("DLCs")
        self.widget.setObjectName("group")
        self.setWidgetResizable(True)
        self.layout = QVBoxLayout()
        self.installed_dlcs = [i.app_name for i in self.core.get_installed_dlc_list()]

        self.installed_dlc_widget = QGroupBox(self.tr("Installed DLCs"))
        self.installed_layout = QVBoxLayout()
        self.available_dlcs = QGroupBox(self.tr("Available DLCs"))
        self.available_dlcs_layout = QVBoxLayout()

        self.installed = []
        self.available = []

    def update_dlcs(self, app_name, dlcs: list):
        self.installed_dlcs = [i.app_name for i in self.core.get_installed_dlc_list()]
        self.installed_dlc_widget = QGroupBox(self.tr("Installed DLCs"))
        self.installed_layout = QVBoxLayout()
        self.available_dlcs = QGroupBox(self.tr("Available DLCs"))
        self.available_dlcs_layout = QVBoxLayout()
        self.installed = []
        self.available = []
        QVBoxLayout().addWidget(self.widget)

        self.game = self.core.get_game(app_name)

        for dlc in sorted(dlcs[self.game.asset_info.catalog_item_id], key=lambda x: x.app_title):
            if dlc.app_name in self.installed_dlcs:
                w = DLCWidget(dlc, True)
                self.installed_layout.addWidget(w)
                self.installed.append(dlc)
            else:
                w = DLCWidget(dlc, False)
                w.install.connect(self.install)
                self.available_dlcs_layout.addWidget(w)
                self.available.append(dlc)
        if len(self.installed) == 0:
            self.installed_layout.addWidget(QLabel(self.tr("No DLCs are installed")))
        if len(self.available) == 0:
            self.available_dlcs_layout.addWidget(QLabel(self.tr("No DLCs are available")))

        self.widget = QGroupBox("DLCs")
        self.layout = QVBoxLayout()
        self.installed_dlc_widget.setLayout(self.installed_layout)
        self.available_dlcs.setLayout(self.available_dlcs_layout)
        self.layout.addWidget(self.installed_dlc_widget)
        self.layout.addWidget(self.available_dlcs)
        self.layout.addStretch(1)
        self.widget.setLayout(self.layout)
        self.setWidget(self.widget)

    def install(self, app_name):
        if not self.core.is_installed(self.game.app_name):
            QMessageBox.warning(self, "Error", self.tr("Base Game is not installed. Please install {} first").format(
                self.game.app_title))
            return
        self.install_dlc.emit(app_name, True)


class DLCWidget(QGroupBox):
    install = pyqtSignal(str)  # Appname

    def __init__(self, dlc: Game, installed: bool):
        super(DLCWidget, self).__init__()
        self.main_layout = QHBoxLayout()
        self.dlc = dlc
        IMAGE_DIR = QSettings().value("img_dir", os.path.expanduser("~/.cache/rare/images"))
        if installed:

            if os.path.exists(os.path.join(IMAGE_DIR, dlc.app_name, "FinalArt.png")):
                pixmap = QPixmap(os.path.join(IMAGE_DIR, dlc.app_name, "FinalArt.png"))
            elif os.path.exists(os.path.join(IMAGE_DIR, dlc.app_name, "DieselGameBoxTall.png")):
                pixmap = QPixmap(os.path.join(IMAGE_DIR, dlc.app_name, "DieselGameBoxTall.png"))
            elif os.path.exists(os.path.join(IMAGE_DIR, dlc.app_name, "DieselGameBoxLogo.png")):
                pixmap = QPixmap(os.path.join(IMAGE_DIR, dlc.app_name, "DieselGameBoxLogo.png"))
            else:
                print(f"No Image found: {dlc.app_title}")
                pixmap = None
            if not pixmap or pixmap.isNull():
                print(dlc.app_title + " has corrupt Image")
                download_image(dlc, force=True)
                pixmap = QPixmap(f"{IMAGE_DIR}/{dlc.app_name}/UninstalledArt.png")
        else:
            if os.path.exists(f"{IMAGE_DIR}/{dlc.app_name}/UninstalledArt.png"):
                pixmap = QPixmap(f"{IMAGE_DIR}/{dlc.app_name}/DieselGameBoxTall.png")

            else:
                pixmap = None

            if not pixmap or pixmap.isNull():
                print(dlc.app_title + " has corrupt Image")
                download_image(dlc, force=True)
                pixmap = QPixmap(f"{IMAGE_DIR}/{dlc.app_name}/UninstalledArt.png")

        image = QLabel()
        image.setPixmap(pixmap)
        self.main_layout.addWidget(image)

        self.layout = QVBoxLayout()

        self.layout.addWidget(QLabel(dlc.app_title))
        self.layout.addWidget(QLabel("Version: " + str(dlc.app_version)))

        self.layout.addWidget(QLabel("App Name: " + dlc.app_name))
        if not installed:
            self.install_button = QPushButton(self.tr("Install"))
            self.layout.addWidget(self.install_button)

            self.install_button.clicked.connect(lambda: self.install.emit(dlc.app_name))
        else:
            self.layout.addWidget(QLabel(self.tr("Installed. Uninstalling DLCs is not supported")))

        self.main_layout.addLayout(self.layout)

        self.setLayout(self.main_layout)

        self.layout.addStretch(1)

    def install_game(self):
        self.install_button.setDisabled(True)
        self.install_button.setText(self.tr("Installing"))
        self.install.emit(self.dlc.app_name)



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'login_dialog.ui'
#
# Created by: PyQt5 UI code generator 5.15.4
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_LoginDialog(object):
    def setupUi(self, LoginDialog):
        LoginDialog.setObjectName("LoginDialog")
        LoginDialog.resize(498, 311)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(LoginDialog.sizePolicy().hasHeightForWidth())
        LoginDialog.setSizePolicy(sizePolicy)
        self.dialog_layout = QtWidgets.QVBoxLayout(LoginDialog)
        self.dialog_layout.setObjectName("dialog_layout")
        spacerItem = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Fixed)
        self.dialog_layout.addItem(spacerItem)
        self.welcome_label = QtWidgets.QLabel(LoginDialog)
        self.welcome_label.setObjectName("welcome_label")
        self.dialog_layout.addWidget(self.welcome_label)
        spacerItem1 = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Fixed)
        self.dialog_layout.addItem(spacerItem1)
        self.login_stack = QtWidgets.QStackedWidget(LoginDialog)
        self.login_stack.setEnabled(True)
        self.login_stack.setMinimumSize(QtCore.QSize(480, 135))
        self.login_stack.setFrameShape(QtWidgets.QFrame.StyledPanel)
        self.login_stack.setFrameShadow(QtWidgets.QFrame.Raised)
        self.login_stack.setObjectName("login_stack")
        self.login_page = QtWidgets.QWidget()
        self.login_page.setEnabled(True)
        self.login_page.setObjectName("login_page")
        self.login_page_layout = QtWidgets.QGridLayout(self.login_page)
        self.login_page_layout.setObjectName("login_page_layout")
        self.login_browser_label = QtWidgets.QLabel(self.login_page)
        font = QtGui.QFont()
        font.setItalic(True)
        self.login_browser_label.setFont(font)
        self.login_browser_label.setObjectName("login_browser_label")
        self.login_page_layout.addWidget(self.login_browser_label, 1, 1, 1, 1)
        self.login_label = QtWidgets.QLabel(self.login_page)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.login_label.setFont(font)
        self.login_label.setObjectName("login_label")
        self.login_page_layout.addWidget(self.login_label, 0, 0, 1, 3, QtCore.Qt.AlignTop)
        self.login_import_label = QtWidgets.QLabel(self.login_page)
        font = QtGui.QFont()
        font.setItalic(True)
        self.login_import_label.setFont(font)
        self.login_import_label.setObjectName("login_import_label")
        self.login_page_layout.addWidget(self.login_import_label, 2, 1, 1, 1)
        self.login_import_radio = QtWidgets.QRadioButton(self.login_page)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.login_import_radio.sizePolicy().hasHeightForWidth())
        self.login_import_radio.setSizePolicy(sizePolicy)
        self.login_import_radio.setObjectName("login_import_radio")
        self.login_page_layout.addWidget(self.login_import_radio, 2, 0, 1, 1)
        self.login_browser_radio = QtWidgets.QRadioButton(self.login_page)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.login_browser_radio.sizePolicy().hasHeightForWidth())
        self.login_browser_radio.setSizePolicy(sizePolicy)
        self.login_browser_radio.setObjectName("login_browser_radio")
        self.login_page_layout.addWidget(self.login_browser_radio, 1, 0, 1, 1)
        spacerItem2 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.login_page_layout.addItem(spacerItem2, 1, 2, 2, 1)
        self.login_stack.addWidget(self.login_page)
        self.success_page = QtWidgets.QWidget()
        self.success_page.setObjectName("success_page")
        self.success_page_layout = QtWidgets.QHBoxLayout(self.success_page)
        self.success_page_layout.setObjectName("success_page_layout")
        self.success_label = QtWidgets.QLabel(self.success_page)
        self.success_label.setObjectName("success_label")
        self.success_page_layout.addWidget(self.success_label)
        self.login_stack.addWidget(self.success_page)
        self.dialog_layout.addWidget(self.login_stack)
        self.button_layout = QtWidgets.QHBoxLayout()
        self.button_layout.setObjectName("button_layout")
        spacerItem3 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.button_layout.addItem(spacerItem3)
        self.exit_button = QtWidgets.QPushButton(LoginDialog)
        self.exit_button.setObjectName("exit_button")
        self.button_layout.addWidget(self.exit_button)
        self.back_button = QtWidgets.QPushButton(LoginDialog)
        self.back_button.setObjectName("back_button")
        self.button_layout.addWidget(self.back_button)
        self.next_button = QtWidgets.QPushButton(LoginDialog)
        self.next_button.setObjectName("next_button")
        self.button_layout.addWidget(self.next_button)
        self.dialog_layout.addLayout(self.button_layout)

        self.retranslateUi(LoginDialog)
        self.login_stack.setCurrentIndex(1)
        QtCore.QMetaObject.connectSlotsByName(LoginDialog)

    def retranslateUi(self, LoginDialog):
        _translate = QtCore.QCoreApplication.translate
        LoginDialog.setWindowTitle(_translate("LoginDialog", "Welcome to Rare"))
        self.welcome_label.setText(_translate("LoginDialog", "<h1>Welcome to Rare</h1>"))
        self.login_browser_label.setText(_translate("LoginDialog", "Login using a browser."))
        self.login_label.setText(_translate("LoginDialog", "Select login method"))
        self.login_import_label.setText(_translate("LoginDialog", "Import from Epic Games Launcher"))
        self.login_import_radio.setText(_translate("LoginDialog", "Import"))
        self.login_browser_radio.setText(_translate("LoginDialog", "Browser"))
        self.success_label.setText(_translate("LoginDialog",
                                              "<html><head/><body><h2 style=\" margin-top:16px; margin-bottom:12px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-size:x-large; font-weight:600;\">Login Successful!</span></h2></body></html>"))
        self.exit_button.setText(_translate("LoginDialog", "Exit"))
        self.back_button.setText(_translate("LoginDialog", "Back"))
        self.next_button.setText(_translate("LoginDialog", "Next"))


if __name__ == "__main__":
    import sys
    app = QtWidgets.QApplication(sys.argv)
    LoginDialog = QtWidgets.QDialog()
    ui = Ui_LoginDialog()
    ui.setupUi(LoginDialog)
    LoginDialog.show()
    sys.exit(app.exec_())



import os

import setuptools

from rare import __version__ as version

with open("README.md", "r") as fh:
    long_description = fh.read()

requirements = [
    "requests<3.0",
    "pillow",
    "setuptools",
    "wheel",
    "PyQt5",
    "QtAwesome",
    "notify-py",
    "psutil",
    "pypresence",
    'pywin32; platform_system == "Windows"'
]

if os.name == "nt":
    requirements.append("pywin32")

setuptools.setup(
    name="Rare",
    version=version,
    author="Dummerle",
    license="GPL-3",
    description="A gui for Legendary",
    long_description=long_description,
    long_description_content_type="text/markdown",
    include_package_data=True,
    url="https://github.com/Dummerle/Rare",
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        'License :: OSI Approved :: GNU General Public License v3 or later (GPLv3+)',
        "Operating System :: OS Independent"
    ],
    python_requires=">=3.8",
    entry_points=dict(console_scripts=["rare=rare.__main__:main"]),
    install_requires=requirements,
)



from logging import getLogger

from PyQt5.QtCore import pyqtSignal
from PyQt5.QtWidgets import QGroupBox
from PyQt5.QtWidgets import QWidget, QCheckBox, QVBoxLayout, QWidgetAction, QMenu, QToolButton, QHBoxLayout

from custom_legendary.core import LegendaryCore
from rare.ui.components.tabs.settings.dxvk import Ui_DxvkSettings

logger = getLogger("DXVK Settings")


class DxvkSettings(QGroupBox, Ui_DxvkSettings):

    def __init__(self, core: LegendaryCore, name=None):
        super(DxvkSettings, self).__init__()
        self.setupUi(self)

        self.name = name if name is not None else "default"
        self.core = core

        self.dxvk_options_map = {
            "devinfo": self.devinfo,
            "fps": self.fps,
            "frametime": self.frametime,
            "memory": self.memory,
            "gpuload": self.gpuload,
            "version": self.version,
            "api": self.api,
        }

        self.load_settings()
        self.show_dxvk.currentIndexChanged.connect(self.store_settings)
        for opt in self.dxvk_options_map:
            self.dxvk_options_map[opt].stateChanged.connect(self.store_settings)

    # Show HUD ComboBox
    # System Default, index 0, removes DXVK_HUD to use setting from env
    # Hidden        , index 1, adds DXVK_HUD=0 to override system configuration
    # Visible       , index 2, adds DXVK_HUD=1 to override system configuration
    # Custom Options, index 3, adds DXVK_HUD=devinfo,fps and enables the customization panel

    def load_settings(self):
        dxvk_options = self.core.lgd.config.get(f"{self.name}.env", "DXVK_HUD", fallback=None)
        self.gb_dxvk_options.setDisabled(True)
        if dxvk_options is not None:
            if dxvk_options == "0":
                self.show_dxvk.setCurrentIndex(1)
            elif dxvk_options == "1":
                self.show_dxvk.setCurrentIndex(2)
            else:
                self.show_dxvk.setCurrentIndex(3)
                self.gb_dxvk_options.setDisabled(False)
                for opt in dxvk_options.split(","):
                    try:
                        self.dxvk_options_map[opt].setChecked(True)
                    except KeyError:
                        print("Malformed DXVK Option: " + opt)
                        continue
        else:
            self.show_dxvk.setCurrentIndex(0)

    def store_settings(self):
        show_dxvk_index = self.show_dxvk.currentIndex()
        if show_dxvk_index:
            if f"{self.name}.env" not in self.core.lgd.config.sections():
                print("add section dxvk")
                self.core.lgd.config.add_section(f"{self.name}.env")
            if show_dxvk_index == 1:
                self.core.lgd.config[f"{self.name}.env"]["DXVK_HUD"] = "0"
            if show_dxvk_index == 2:
                self.core.lgd.config[f"{self.name}.env"]["DXVK_HUD"] = "1"
            if show_dxvk_index == 3:
                dxvk_options = []
                for opt in self.dxvk_options_map:
                    if self.dxvk_options_map[opt].isChecked():
                        dxvk_options.append(opt)
                if not dxvk_options:
                    # Check if this is the first activation
                    stored = self.core.lgd.config.get(f"{self.name}.env", "DXVK_HUD", fallback=None)
                    if stored not in [None, "0", "1"]:
                        self.core.lgd.config[f"{self.name}.env"]["DXVK_HUD"] = "0"
                    else:
                        dxvk_options = ["devinfo", "fps"]
                # Check again if dxvk_options changed due to first activation
                if dxvk_options:
                    self.core.lgd.config[f"{self.name}.env"]["DXVK_HUD"] = ",".join(dxvk_options)
        else:
            if self.core.lgd.config.get(f"{self.name}.env", "DXVK_HUD", fallback=None) is not None:
                self.core.lgd.config.remove_option(f"{self.name}.env", "DXVK_HUD")
                if not self.core.lgd.config[f"{self.name}.env"]:
                    self.core.lgd.config.remove_section(f"{self.name}.env")
        self.core.lgd.save_config()
        self.load_settings()


class DxvkWidget(QGroupBox):

    def __init__(self, core: LegendaryCore, name=None):
        super(DxvkWidget, self).__init__()
        self.core = core
        self.setObjectName("settings_widget")
        self.dxvk_settings = {
            "fps": [False, "Fps"],
            "gpuload": [False, self.tr("GPU usage")],
            "memory": [False, self.tr("Used Memory")],
            "devinfo": [False, self.tr("Device info")],
            "version": [False, self.tr("DXVK version")],
            "api": [False, self.tr("D3D Level of application")],
            "frametime": [False, self.tr("Frame time graph")]
        }
        self.name = name
        self.layout = QVBoxLayout()
        self.child_layout = QHBoxLayout()
        self.setTitle(self.tr("dxvk settings"))
        self.show_dxvk = QCheckBox("Show Dxvk HUD")

        self.more_settings = QToolButton()
        dxvk_hud = self.core.lgd.config.get(f"{self.name}.env", "DXVK_HUD", fallback="")

        self.more_settings.setDisabled(not dxvk_hud == "")
        if dxvk_hud:
            for s in dxvk_hud.split(","):
                self.dxvk_settings[s][0] = True

        self.more_settings.setPopupMode(QToolButton.InstantPopup)
        self.more_settings.setMenu(QMenu())
        self.more_settings.setText("More DXVK settings")

        action = QWidgetAction(self)
        self.more_settings_widget = DxvkMoreSettingsWidget(self.dxvk_settings, self.core)
        self.more_settings_widget.show_dxvk.connect(lambda x: self.show_dxvk.setChecked(x))
        action.setDefaultWidget(self.more_settings_widget)
        self.more_settings.menu().addAction(action)

        self.show_dxvk.stateChanged.connect(self.update_dxvk_active)
        self.show_dxvk.setChecked(not dxvk_hud == "")
        self.child_layout.addWidget(self.show_dxvk)

        self.child_layout.addWidget(self.more_settings)
        self.layout.addLayout(self.child_layout)

        self.setLayout(self.layout)

    def update_settings(self, app_name):
        self.name = app_name
        dxvk_hud = self.core.lgd.config.get(f"{self.name}.env", "DXVK_HUD", fallback="")
        if dxvk_hud:
            self.more_settings.setDisabled(False)
            for s in dxvk_hud.split(","):
                self.dxvk_settings[s][0] = True
        else:
            self.show_dxvk.setChecked(False)
            self.more_settings.setDisabled(True)

    def update_dxvk_active(self):
        if self.show_dxvk.isChecked():
            if not f"{self.name}.env" in self.core.lgd.config.sections():
                print("add section dxvk")
                self.core.lgd.config.add_section(f"{self.name}.env")
            self.more_settings.setDisabled(False)

            for i in self.more_settings_widget.settings:
                if i in ["fps", "gpuload"]:
                    self.more_settings_widget.settings[i][0] = True

            self.core.lgd.config[f"{self.name}.env"]["DXVK_HUD"] = "fps,gpuload"
            for w in self.more_settings_widget.widgets:
                if w.tag == "fps" or w.tag == "gpuload":
                    w.setChecked(True)

                else:
                    w.setChecked(False)
        else:
            self.more_settings.setDisabled(True)
            if not self.core.lgd.config.get(f"{self.name}.env", "DXVK_HUD", fallback="") == "":
                self.core.lgd.config.remove_option(f"{self.name}.env", "DXVK_HUD")
                if not self.core.lgd.config[f"{self.name}.env"]:
                    self.core.lgd.config.remove_section(f"{self.name}.env")
        self.core.lgd.save_config()


class DxvkMoreSettingsWidget(QWidget):
    show_dxvk = pyqtSignal(bool)

    def __init__(self, settings: dict, core: LegendaryCore):
        super(DxvkMoreSettingsWidget, self).__init__()
        self.layout = QVBoxLayout()
        self.name = "default"
        self.widgets = []
        self.core = core
        self.settings = settings
        for i in settings:
            widget = CheckBox(i, settings[i])
            widget.signal.connect(self.change)
            self.layout.addWidget(widget)
            self.widgets.append(widget)

        self.setLayout(self.layout)

    def change(self, signal: list):
        tag, checked = signal
        self.settings[tag][0] = checked

        sett = []
        logger.debug(self.settings)
        for i in self.settings:
            check, _ = self.settings[i]
            if check:
                sett.append(i)
        if len(sett) > 0:
            self.core.lgd.config.set(f"{self.name}.env", "DXVK_HUD", ",".join(sett))

        else:
            self.core.lgd.config.remove_option(f"{self.name}.env", "DXVK_HUD")
            self.show_dxvk.emit(False)
            if not self.core.lgd.config.options(f"{self.name}.env"):
                self.core.lgd.config.remove_section(f"{self.name}.env")
        self.core.lgd.save_config()


class CheckBox(QCheckBox):
    signal = pyqtSignal(tuple)

    def __init__(self, tag, settings):
        checked, text = settings
        super(CheckBox, self).__init__(text)
        self.setChecked(checked)
        self.tag = tag
        self.clicked.connect(lambda: self.signal.emit((self.tag, self.isChecked())))



from logging import getLogger

from PyQt5.QtWidgets import QFileDialog, QWidget

from custom_legendary.core import LegendaryCore
from rare.components.tabs.settings.dxvk import DxvkSettings, DxvkWidget
from rare.ui.components.tabs.settings.linux import Ui_LinuxSettings
from rare.utils.extra_widgets import PathEdit

logger = getLogger("LinuxSettings")


class LinuxSettings(QWidget, Ui_LinuxSettings):
    def __init__(self, core: LegendaryCore, name=None):
        super(LinuxSettings, self).__init__()
        self.setupUi(self)

        self.name = name if name is not None else "default"
        self.core = core

        # Wine prefix
        self.wine_prefix = PathEdit(self.core.lgd.config.get(self.name, "wine_prefix", fallback=""),
                                    file_type=QFileDialog.DirectoryOnly,
                                    save_func=lambda: self.save_setting(self.wine_prefix, "wine_prefix"))
        self.prefix_layout.addWidget(self.wine_prefix)

        # Wine executable
        self.wine_exec = PathEdit(self.core.lgd.config.get(self.name, "wine_executable", fallback=""),
                                  file_type=QFileDialog.ExistingFile,
                                  name_filter="Wine executable (wine wine64)",
                                  save_func=lambda: self.save_setting(self.wine_exec, "wine_executable"))
        self.exec_layout.addWidget(self.wine_exec)

        # dxvk
        # FIXME: Remove this check when done with per game settings
        if name is None:
            self.dxvk = DxvkSettings(core, self.name)
        else:
            self.dxvk = DxvkWidget(core)
        self.dxvk_layout.addWidget(self.dxvk)

    def save_setting(self, widget: PathEdit, setting_name: str):
        if self.name not in self.core.lgd.config.sections():
            self.core.lgd.config.add_section(self.name)

        self.core.lgd.config.set(self.name, setting_name, widget.text())
        if not widget.text():
            self.core.lgd.config.remove_option(self.name, setting_name)
        else:
            logger.info("Set config of wine_prefix to " + widget.text())
        if self.core.lgd.config[self.name] == {}:
            self.core.lgd.config.remove_section(self.name)
        self.core.lgd.save_config()



import os
from getpass import getuser
from logging import getLogger

from PyQt5.QtCore import pyqtSignal
from PyQt5.QtWidgets import QWidget, QFileDialog

from custom_legendary.core import LegendaryCore
from rare.ui.components.dialogs.login.import_login import Ui_ImportLogin

logger = getLogger("ImportLogin")


class ImportLogin(QWidget, Ui_ImportLogin):
    success = pyqtSignal()
    changed = pyqtSignal()
    if os.name == "nt":
        localappdata = os.path.expandvars("%LOCALAPPDATA%")
    else:
        localappdata = os.path.join("drive_c/users", getuser(), "Local Settings/Application Data")
    appdata_path = os.path.join(localappdata, "EpicGamesLauncher/Saved/Config/Windows")
    found = False

    def __init__(self, core: LegendaryCore, parent=None):
        super(ImportLogin, self).__init__(parent=parent)
        self.setupUi(self)

        self.core = core

        self.text_egl_found = self.tr("Found EGL Program Data. Click 'Next' to import them.")
        self.text_egl_notfound = self.tr("Could not find EGL Program Data. ")

        if os.name == "nt":
            if not self.core.egl.appdata_path and os.path.exists(self.egl_data_path):
                self.core.egl.appdata_path = self.appdata_path
            if not self.core.egl.appdata_path:
                self.status_label.setText(self.text_egl_notfound)
            else:
                self.status_label.setText(self.text_egl_found)
                self.found = True
        else:
            self.info_label.setText(self.tr(
                "Please select the Wine prefix"
                " where Epic Games Launcher is installed. ") + self.info_label.text()
                                    )
            prefixes = self.get_wine_prefixes()
            if len(prefixes):
                self.prefix_combo.addItems(prefixes)
                self.status_label.setText(self.tr("Select the Wine prefix you want to import."))
            else:
                self.status_label.setText(self.text_egl_notfound)

        self.prefix_tool.clicked.connect(self.prefix_path)
        self.prefix_combo.editTextChanged.connect(self.changed.emit)

    def get_wine_prefixes(self):
        possible_prefixes = [
            os.path.expanduser("~/.wine"),
            os.path.expanduser("~/Games/epic-games-store"),
        ]
        prefixes = []
        for prefix in possible_prefixes:
            if os.path.exists(os.path.join(prefix, self.appdata_path)):
                prefixes.append(prefix)
        return prefixes

    def prefix_path(self):
        prefix_dialog = QFileDialog(self, self.tr("Choose path"), os.path.expanduser("~/"))
        prefix_dialog.setFileMode(QFileDialog.DirectoryOnly)
        if prefix_dialog.exec_():
            names = prefix_dialog.selectedFiles()
            self.prefix_combo.setCurrentText(names[0])

    def is_valid(self):
        if os.name == "nt":
            return self.found
        else:
            return os.path.exists(os.path.join(self.prefix_combo.currentText(), self.appdata_path))

    def do_login(self):
        self.status_label.setText(self.tr("Loading..."))
        if os.name == "nt":
            pass
        else:
            self.core.egl.appdata_path = os.path.join(self.prefix_combo.currentText(), self.appdata_path)
        try:
            if self.core.auth_import():
                logger.info(f"Logged in as {self.core.lgd.userdata['displayName']}")
                self.success.emit()
            else:
                self.status_label.setText(self.tr("Login failed."))
                logger.warning("Failed to import existing session.")
        except Exception as e:
            self.status_label.setText(self.tr("Login failed. ") + str(e))
            logger.warning("Failed to import existing session: " + str(e))



import os

__version__ = "1.5.0"

resources_path = os.path.join(os.path.dirname(__file__), "resources")
languages_path = os.path.join(os.path.dirname(__file__), "languages")

# Cache Directory: Store images
if p := os.getenv("XDG_CACHE_HOME"):
    cache_dir = os.path.join(p, "rare/cache")
elif os.name == "nt":
    cache_dir = os.path.expandvars("%APPDATA%/rare/cache")
else:
    cache_dir = os.path.expanduser("~/.cache/rare/")
if not os.path.exists(cache_dir):
    os.makedirs(cache_dir)

# Data Directory: Images
if p := os.getenv("XDG_DATA_HOME"):
    data_dir = os.path.join(p, "rare")
if os.name == "nt":
    data_dir = os.path.expandvars("%APPDATA%/rare/")
else:
    data_dir = os.path.expanduser("~/.local/share/rare/")
if not os.path.exists(data_dir):
    os.makedirs(data_dir)

image_dir = os.path.join(data_dir, "images")



import os
from multiprocessing import Queue as MPQueue

from PyQt5.QtCore import Qt, QObject, QRunnable, QThreadPool, pyqtSignal, pyqtSlot
from PyQt5.QtGui import QCloseEvent
from PyQt5.QtWidgets import QDialog, QFileDialog, QCheckBox, QMessageBox

from custom_legendary.core import LegendaryCore
from custom_legendary.utils.selective_dl import games
from rare.ui.components.dialogs.install_dialog import Ui_InstallDialog
from rare.utils.extra_widgets import PathEdit
from rare.utils.models import InstallDownloadModel, InstallQueueItemModel
from rare.utils.utils import get_size


class InstallDialog(QDialog, Ui_InstallDialog):
    result_ready = pyqtSignal(InstallQueueItemModel)

    def __init__(self, core: LegendaryCore, dl_item: InstallQueueItemModel, update=False, silent=False, parent=None):
        super(InstallDialog, self).__init__(parent)
        self.setupUi(self)
        self.setAttribute(Qt.WA_DeleteOnClose, True)
        self.setWindowFlags(Qt.Dialog | Qt.CustomizeWindowHint | Qt.WindowTitleHint)

        self.core = core
        self.dl_item = dl_item
        self.dl_item.status_q = MPQueue()
        self.app_name = self.dl_item.options.app_name
        self.game = self.core.get_game(self.app_name)
        self.update = update
        self.silent = silent

        self.threadpool = QThreadPool(self)
        self.threadpool.setMaxThreadCount(1)

        header = self.tr("Update") if update else self.tr("Install")
        self.install_dialog_label.setText(f"<h3>{header} \"{self.game.app_title}\"</h3>")
        self.setWindowTitle(f"{self.windowTitle()} - {header} \"{self.game.app_title}\"")

        if self.core.lgd.config.has_option("Legendary", "install_dir"):
            default_path = self.core.lgd.config.get("Legendary", "install_dir")
        else:
            default_path = os.path.expanduser("~/legendary")
        if not default_path:
            default_path = os.path.expanduser("~/legendary")

        self.install_dir_edit = PathEdit(text=default_path,
                                         file_type=QFileDialog.DirectoryOnly,
                                         edit_func=self.option_changed)
        self.install_dir_layout.addWidget(self.install_dir_edit)

        if self.update:
            self.install_dir_label.setVisible(False)
            self.install_dir_edit.setVisible(False)

        self.warn_label.setVisible(False)
        self.warn_message.setVisible(False)

        if self.core.lgd.config.has_option("Legendary", "max_workers"):
            max_workers = self.core.lgd.config.get("Legendary", "max_workers")
        else:
            max_workers = 0
        self.max_workers_spin.setValue(int(max_workers))
        self.max_workers_spin.valueChanged.connect(self.option_changed)

        self.force_download_check.stateChanged.connect(self.option_changed)
        self.ignore_space_check.stateChanged.connect(self.option_changed)
        self.download_only_check.stateChanged.connect(self.option_changed)

        self.sdl_list_checks = list()
        try:
            for key, info in games[self.app_name].items():
                cb = QDataCheckBox(info['name'], info['tags'])
                if key == '__required':
                    self.dl_item.options.sdl_list.extend(info['tags'])
                    cb.setChecked(True)
                    cb.setDisabled(True)
                self.sdl_list_layout.addWidget(cb)
                self.sdl_list_checks.append(cb)
            self.sdl_list_frame.resize(self.sdl_list_frame.minimumSize())
            for cb in self.sdl_list_checks:
                cb.stateChanged.connect(self.option_changed)
        except KeyError:
            self.sdl_list_frame.setVisible(False)
            self.sdl_list_label.setVisible(False)

        self.install_button.setEnabled(False)

        self.cancel_button.clicked.connect(self.cancel_clicked)
        self.verify_button.clicked.connect(self.verify_clicked)
        self.install_button.clicked.connect(self.install_clicked)

        self.options_changed = False
        self.worker_running = False
        self.reject_close = True

        self.resize(self.minimumSize())
        # self.setFixedSize(self.size())

        self.verify_clicked()

    def execute(self):
        if self.silent:
            self.reject_close = False
            self.get_download_info()
        else:
            self.show()

    def get_options(self):
        self.dl_item.options.base_path = self.install_dir_edit.text() if not self.update else None
        self.dl_item.options.max_workers = self.max_workers_spin.value()
        self.dl_item.options.force = self.force_download_check.isChecked()
        self.dl_item.options.ignore_space_req = self.ignore_space_check.isChecked()
        self.dl_item.options.no_install = self.download_only_check.isChecked()
        self.dl_item.options.sdl_list = ['']
        for cb in self.sdl_list_checks:
            if data := cb.isChecked():
                self.dl_item.options.sdl_list.extend(data)

    def get_download_info(self):
        self.dl_item.download = None
        info_worker = InstallInfoWorker(self.core, self.dl_item)
        info_worker.setAutoDelete(True)
        info_worker.signals.result.connect(self.on_worker_result)
        info_worker.signals.failed.connect(self.on_worker_failed)
        info_worker.signals.finished.connect(self.on_worker_finished)
        self.worker_running = True
        self.threadpool.start(info_worker)

    def verify_clicked(self):
        message = self.tr("Updating...")
        self.download_size_info_label.setText(message)
        self.download_size_info_label.setStyleSheet("font-style: italic; font-weight: normal")
        self.install_size_info_label.setText(message)
        self.install_size_info_label.setStyleSheet("font-style: italic; font-weight: normal")
        self.cancel_button.setEnabled(False)
        self.verify_button.setEnabled(False)
        self.install_button.setEnabled(False)
        self.options_changed = False
        self.get_options()
        self.get_download_info()

    def option_changed(self):
        self.options_changed = True
        self.install_button.setEnabled(False)
        self.verify_button.setEnabled(not self.worker_running)

    def cancel_clicked(self):
        self.dl_item.download = None
        self.reject_close = False
        self.close()

    def install_clicked(self):
        self.reject_close = False
        self.close()

    def on_worker_result(self, dl_item: InstallDownloadModel):
        self.dl_item.download = dl_item
        download_size = self.dl_item.download.analysis.dl_size
        install_size = self.dl_item.download.analysis.install_size
        if download_size:
            self.download_size_info_label.setText("{}".format(get_size(download_size)))
            self.download_size_info_label.setStyleSheet("font-style: normal; font-weight: bold")
            self.install_button.setEnabled(not self.options_changed)
        else:
            self.install_size_info_label.setText(self.tr("Game already installed"))
            self.install_size_info_label.setStyleSheet("font-style: italics; font-weight: normal")
        self.install_size_info_label.setText("{}".format(get_size(install_size)))
        self.install_size_info_label.setStyleSheet("font-style: normal; font-weight: bold")
        self.verify_button.setEnabled(self.options_changed)
        self.cancel_button.setEnabled(True)
        if self.silent:
            self.close()

    def on_worker_failed(self, message: str):
        error_text = self.tr("Error")
        self.download_size_info_label.setText(error_text)
        self.install_size_info_label.setText(error_text)
        QMessageBox.critical(self, self.windowTitle(), message)
        self.verify_button.setEnabled(self.options_changed)
        self.cancel_button.setEnabled(True)
        if self.silent:
            self.close()

    def on_worker_finished(self):
        self.worker_running = False

    # lk: happens when close() is called, also when top right 'X' is pressed.
    # lk: reject any events not coming from the buttons in case the WM
    # lk: doesn't honor the window hints
    def closeEvent(self, a0: QCloseEvent) -> None:
        if self.reject_close:
            a0.ignore()
        else:
            self.threadpool.clear()
            self.threadpool.waitForDone()
            self.result_ready.emit(self.dl_item)
            a0.accept()


class InstallInfoWorkerSignals(QObject):
    result = pyqtSignal(InstallDownloadModel)
    failed = pyqtSignal(str)
    finished = pyqtSignal()


class InstallInfoWorker(QRunnable):

    def __init__(self, core: LegendaryCore, dl_item: InstallQueueItemModel):
        super(InstallInfoWorker, self).__init__()
        self.signals = InstallInfoWorkerSignals()
        self.core = core
        self.dl_item = dl_item

    @pyqtSlot()
    def run(self):
        try:
            download = InstallDownloadModel(*self.core.prepare_download(
                app_name=self.dl_item.options.app_name,
                base_path=self.dl_item.options.base_path,
                force=self.dl_item.options.force,
                no_install=self.dl_item.options.no_install,
                status_q=self.dl_item.status_q,
                # max_shm=,
                max_workers=self.dl_item.options.max_workers,
                # game_folder=,
                # disable_patching=,
                # override_manifest=,
                # override_old_manifest=,
                # override_base_url=,
                # platform_override=,
                # file_prefix_filter=,
                # file_exclude_filter=,
                # file_install_tag=,
                # dl_optimizations=,
                # dl_timeout=,
                repair=self.dl_item.options.repair,
                # repair_use_latest=,
                ignore_space_req=self.dl_item.options.ignore_space_req,
                # disable_delta=,
                # override_delta_manifest=,
                # reset_sdl=,
                sdl_prompt=lambda app_name, title: self.dl_item.options.sdl_list
            ))
            if not download.res.failures:
                self.signals.result.emit(download)
            else:
                self.signals.failed.emit("\n".join(str(i) for i in download.res.failures))
        except Exception as e:
            self.signals.failed.emit(str(e))
        self.signals.finished.emit()


class QDataCheckBox(QCheckBox):

    def __init__(self, text, data=None, parent=None):
        super(QDataCheckBox, self).__init__(parent)
        self.setText(text)
        self.data = data

    def isChecked(self):
        if super(QDataCheckBox, self).isChecked():
            return self.data
        else:
            return False



from logging import getLogger

from PyQt5.QtCore import pyqtSignal
from PyQt5.QtWidgets import QGroupBox

logger = getLogger("Uninstalled")


class BaseUninstalledWidget(QGroupBox):
    show_uninstalled_info = pyqtSignal(str)

    def __init__(self, game, core, pixmap):
        super(BaseUninstalledWidget, self).__init__()
        self.game = game
        self.core = core
        self.pixmap = pixmap
        self.installing = False

        self.setContentsMargins(0, 0, 0, 0)

    def install(self):
        self.show_uninstalled_info.emit(self.game.app_name)
